local addonName, vars = ...
local L = vars.L
AscendanceTimer = vars
local addon = vars

local settings

local revision = tonumber(("$Revision: 13 $"):match("%d+"))
local LDB, LDBo

local SPELLID_ASCENDANCE = 114049
local SPELLID_LAVABURST = 51505
local SPELLID_LAVABURST_OVERLOAD = 77451
local NAME_LB = GetSpellInfo(SPELLID_LAVABURST)
local NAME_ASCENDANCE, _, TEX_ASCENDANCE = GetSpellInfo(SPELLID_ASCENDANCE)

local DURATION = 15
local NOMINALCD = 3*60
local SETREDUCE = 1.5   -- tooltip is WRONG: it's actually 1.5 sec per LvB
local MINLEVEL = 87
local playerGUID

local updatebuttons = true -- workaround the bugs in the default UI CD display

addon.expires = 0
addon.fightstat = {} -- per-fight counters      (spells and states)
addon.cdstat = {}    -- per-ascendance counters (spells)
addon.buttons = {}
addon.lastascend = 0
addon.lastevent = 0
addon.combatstart = 0

local function chatMsg(msg)
     DEFAULT_CHAT_FRAME:AddMessage("|cffffff40"..addonName.."|r: "..msg)
end

local function debug(msg)
  if settings and settings.debug then
     chatMsg("(debug) "..msg)
  end
end

local function statMsg(msg)
  if settings and settings.stats then
     chatMsg(msg)
  else
     debug(msg)
  end
end

local function timefmt(t)
  t = t or 0
  local min = math.floor(t / 60)
  local sec = math.floor(t % 60)
  return string.format("%d:%02d",min,sec)
end

function addon:Update()
  if UnitLevel("player") < MINLEVEL then
    LDBo.text = LEVEL
    return
  end
  local now = GetTime()
  local start, duration = GetSpellCooldown(SPELLID_ASCENDANCE)
  if start and duration and start > 0 and duration > 0 then
    local exp = start + duration
    if math.abs(exp - addon.expires) > 3 and exp >= now then
      debug("Adjusting CD: "..string.format("%0.2f",exp-addon.expires).." sec")
      addon.expires = exp
    end
  end
  local remain = addon.expires - now
  if remain <= 0 then
    LDBo.text = READY
    if not addon.ready and addon.lastascend > 0 then -- cooldown completed
      local cdtime = now - addon.lastascend
      local msg = L["Ascendance cooldown completed after: "]..timefmt(cdtime)..
         "  ("..(addon.cdstat[SPELLID_LAVABURST] or 0).." "..NAME_LB..", "..
                (addon.cdstat[SPELLID_LAVABURST_OVERLOAD] or 0).." "..L["Overload"]..")"
      statMsg(msg)
      addon.fightstat.oncd = (addon.fightstat.oncd or 0) + (now - addon.lastevent)
      addon.lastevent = now
      addon.brokemsg = nil
    end
    addon.ready = true
  else
    LDBo.text = timefmt(remain)
    addon.ready = false
    -- finished ascension, now on cd.  This assumes you don't click off the buff, which would be silly
    if addon.lastevent == addon.lastascend and now - addon.lastascend >= DURATION then 
      debug("Ascendance over")
      addon.fightstat.incd = (addon.fightstat.incd or 0) + (now - addon.lastevent)
      addon.lastevent = now
    end
    if settings and settings.debug and not addon.brokemsg and
       start == 0 and now - addon.lastascend >= DURATION + 2 and remain > 10 then
      local t,a = issecurevariable("GetSpellCooldown")
      debug("GetSpellCooldown() is returning incorrect results! ("..tostring(t)..","..tostring(a)..")")
      addon.brokemsg = true
    end
  end
end

local function OnUpdate(_, elap)
  addon.elap = (addon.elap or 0) - elap
  if addon.elap < 0 then
     addon.elap = 0.25
     addon:Update()
  end
end

local tier_slots = { INVSLOT_HEAD, INVSLOT_SHOULDER, INVSLOT_CHEST, INVSLOT_LEGS, INVSLOT_HAND }
local tier_pieces = {
  [95951]=1,[95950]=1,[95952]=1,[95953]=1,[95954]=1,
  [95321]=1,[95320]=1,[95322]=1,[95323]=1,[95324]=1,
  [96695]=1,[96694]=1,[96696]=1,[96697]=1,[96698]=1,
}

function addon:check_tier()
    local cnt = 0
    for _, slot in pairs(tier_slots) do
       local itemid = GetInventoryItemID("player", slot)
       if itemid and tier_pieces[itemid] then
         cnt = cnt + 1
       end
    end
    addon.T15set = (cnt >= 4)
    if addon.T15set then
      debug("T15 set bonus enabled")
    else
      debug("T15 set bonus disabled")
    end
end

local actionbars = { 
  "Action", 
  "MultiBarBottomLeft",
  "MultiBarBottomRight",
  "MultiBarRight",
  "MultiBarLeft",
}

function addon:check_buttons()
  if not updatebuttons then return end
  wipe(addon.buttons)
  for _,bar in ipairs(actionbars) do
    for slot = 1, NUM_ACTIONBAR_BUTTONS do
      local button = _G[bar.."Button"..slot]
      local action = button and button.action
      if action and button.cooldown then
        local type, id  = GetActionInfo(action)
        if (type == "spell" and id == SPELLID_ASCENDANCE) or 
           (type == "macro" and GetActionTexture(action) == TEX_ASCENDANCE) then
            debug(button:GetName())
            addon.buttons[button.cooldown] = button
        end
      end
    end
  end
end

function addon:update_buttons()
  if not updatebuttons then return end
  for cooldown, button in pairs(addon.buttons) do
    --debug(event.." Updating button: "..button:GetName())
    if addon.ready then
      CooldownFrame_SetTimer(cooldown, 0, 0, true)
    else
      CooldownFrame_SetTimer(cooldown, addon.expires - NOMINALCD, NOMINALCD, true)
    end
  end
end

local function OnEvent(_, event, ...)
  if event == "COMBAT_LOG_EVENT_UNFILTERED" then
    local _, subevent, _, srcGUID, _, _, _, dstGUID = ...
    if srcGUID == playerGUID and (subevent == "SPELL_DAMAGE" or subevent == "SPELL_MISSED") then
      if spellid == SPELLID_LAVABURST or spellid == SPELLID_LAVABURST_OVERLOAD then
        --debug(spellid)
	addon.cdstat[spellid] = (addon.cdstat[spellid] or 0) + 1
	addon.fightstat[spellid] = (addon.fightstat[spellid] or 0) + 1
        if addon.T15set then
          addon.expires = addon.expires - SETREDUCE
          addon:Update()
          addon:update_buttons()
        end
      end
    elseif dstGUID == playerGUID and subevent == "SPELL_AURA_APPLIED" then
      local spellid = select(12, ...)
      -- 8/19 20:19:22.250  SPELL_AURA_APPLIED,0xF130EE47000005AF,"Sha of Fear",0xa48,0x0,0x06800000004CDB68,"Ashel",0x511,0x0,129378,"Fading Light",0x1,0x06800000004CDB68,575075,2531,38126,0,300000,BUFF
      if spellid == 129378 -- Sha of Fear Fading Light resets cd
      or spellid == 141681 -- Dark Heart of Pandaria Orb of Power
      -- or spellid == 324    -- Lightning Shield (testing)
	 then 
	debug("Resetting CDs: "..GetSpellInfo(spellid).." "..spellid)
        addon.expires = GetTime()
        addon:Update()
        addon:update_buttons()
      end
    end
  elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
    local unit, spellname, _, _, spellid = ...
    if unit ~= "player" then return end
    if spellid == SPELLID_ASCENDANCE then
      local now = GetTime()
      debug(spellname)
      addon.expires = now + NOMINALCD
      addon.lastascend = now
      addon.fightstat.offcd = (addon.fightstat.offcd or 0) + (now - addon.lastevent)
      wipe(addon.cdstat)
      addon.lastevent = now
      addon:Update()
    end
  elseif event == "PLAYER_REGEN_DISABLED" then -- entering combat
    local now = GetTime()
    addon:check_tier()
    addon:check_buttons()
    addon.lastevent = now
    addon.combatstart = now
    wipe(addon.fightstat)
  elseif event == "PLAYER_REGEN_ENABLED" then -- leaving combat
    if addon.lastascend < addon.combatstart then return end -- didn't use ascendance at all that fight, skip fightstat output
    local now = GetTime()
    local s = addon.fightstat
    local state = (addon.ready and "offcd" or ((addon.lastascend == addon.lastevent) and "incd" or "oncd"))
    s[state] = (s[state] or 0) + (now - addon.lastevent)
    local combattime = (now - addon.combatstart)
    local msg = L["Uptime"]..": |cff00ff00"..timefmt(s.incd).. string.format(" (%0.2f%%)|r",100*(s.incd or 0)/combattime)..
         "  ".. L["On cooldown"]..": |cff8080ff".. timefmt(s.oncd).. string.format(" (%0.2f%%)|r",100*(s.oncd or 0)/combattime)..
         "  ".. L["Off cooldown"]..": |cffff0000"..timefmt(s.offcd)..string.format(" (%0.2f%%)|r",100*(s.offcd or 0)/combattime)
    local msg2 = (s[SPELLID_LAVABURST] or 0).." "..NAME_LB..", "..
                 (s[SPELLID_LAVABURST_OVERLOAD] or 0).." "..L["Overload"];
    statMsg(msg)
    statMsg(msg2)
  elseif event == "ADDON_LOADED" and select(1,...) == addonName then
    addon:OnEnable()
  elseif event == "ACTIONBAR_UPDATE_COOLDOWN" then
    addon:update_buttons()
  end
end

if select(2,UnitClass("player")) ~= "SHAMAN" then
   return
end

local hiddenFrame = CreateFrame("Button", addonName.."HiddenFrame", UIParent)
hiddenFrame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
hiddenFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
hiddenFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
hiddenFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
hiddenFrame:RegisterEvent("ADDON_LOADED")
if updatebuttons then
  hiddenFrame:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")
end
hiddenFrame:SetScript("OnUpdate",OnUpdate)
hiddenFrame:SetScript("OnEvent",OnEvent)

function addon:OnEnable()
  debug("OnEnable")
  AscendanceTimerDB = AscendanceTimerDB or {}
  settings = AscendanceTimerDB
  if settings.stats == nil then settings.stats = true end


  playerGUID = UnitGUID("player")
 
  if LDB then
    return
  end
  LDB = LibStub:GetLibrary("LibDataBroker-1.1")
  LDBo = LDB:NewDataObject(addonName, {
        type = "data source",
        label = addonName,
	text = READY,
        icon = "Interface\\Icons\\spell_fire_elementaldevastation",
        OnClick = function(self, button)
		local set
                if button == "RightButton" then
			set = "debug"
                else
			set = "stats"
                end
		settings[set] = not settings[set]
		chatMsg(set.." "..(settings[set] and L["enabled"] or L["disabled"]))
        end,
        OnTooltipShow = function(tooltip)
                if tooltip and tooltip.AddLine then
                        tooltip:SetText(addonName)
                        tooltip:AddLine("|cffff8040"..L["Left Click"].."|r "..L["to toggle cooldown stats"])
                        tooltip:AddLine("|cffff8040"..L["Right Click"].."|r "..L["to toggle debugging"])
                        tooltip:Show()
                end
        end,
     })

  addon:check_tier()
  addon:check_buttons()
  addon:Update()
end

